<?php

namespace Drupal\warden\Service;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Extension\InfoParser;
use WardenApi\Api;
use Drupal\Core\Extension\Extension;

/**
 * Default controller for the warden module.
 */
class WardenManager extends Api {

  /**
   * @var InfoParser
   */
  protected $infoParser;

  /**
   * @var string
   */
  protected $siteName;

  /**
   * @var string
   */
  protected $baseUrl;

  /**
   * @var string
   */
  protected $coreVersion;

  /**
   * @var Extension[]
   */
  protected $themes;

  /**
   * @var Extension[]
   */
  protected $modules;

  /**
   * @var bool
   */
  protected $includeContrib;

  /**
   * @var bool
   */
  protected $includeCustom;

  /**
   * @var string
   */
  protected $customModuleRegex;

  /**
   * @var string
   */
  protected $contribModuleRegex;

  /**
   * @var string
   */
  protected $localToken;

  /**
   * @var int
   */
  protected $time;

  /**
   * @param ConfigFactory $configFactory
   */
  public function __construct(ConfigFactory $configFactory) {
    $warden_settings = $configFactory->get('warden.settings');

    $this->siteName = $configFactory->get('system.site')->get('name');
    $this->contribModuleRegex = $warden_settings->get('warden_preg_match_contrib');
    $this->customModuleRegex = $warden_settings->get('warden_preg_match_custom');
    $this->includeCustom = $warden_settings->get('warden_match_custom');
    $this->includeContrib = $warden_settings->get('warden_match_contrib');

    $local_token = $warden_settings->get('warden_token');
    if (!empty($local_token)) {
      $this->localToken = $local_token;
    }

    $this->coreVersion = \Drupal::VERSION;

    parent::__construct(
      $warden_settings->get('warden_server_host_path'),
      $warden_settings->get('warden_http_username'),
      $warden_settings->get('warden_http_password'),
      $warden_settings->get('warden_certificate_path')
    );
  }

  /**
   * Generate all the site's data for Warden.
   *
   * @return array
   *   The site's data as an array.
   */
  public function generateSiteData() {
    $result = [
      'core' => [
        'drupal' => [
          'version' => $this->getCoreVersion(),
        ],
      ],
      'contrib' => [],
      'custom' => [],
      'url' => $this->getBaseUrl(),
      'site_name' => $this->getSiteName(),
      'key' => $this->getLocalToken(),
      'time' => $this->getTime(),
    ];

    // Indicate if we are not including custom modules.
    if (!$this->includeCustomModules()) {
      $result['custom'] = 'disabled';
    }

    // Indicate if we are not including contrib modules.
    if (!$this->includeContribModules()) {
      $result['contrib'] = 'disabled';
    }

    // Include all contrib themes.
    if ($this->includeContribModules()) {
      foreach ($this->getThemes() as $theme) {
        if (isset($theme->info['package']) && $theme->info['package'] == 'Core') {
          continue;
        }

        if (isset($theme->info['version'])) {
          $result['contrib'][$theme->name] = [
            'version' => $theme->info['version'],
          ];
        }
      }
    }

    // Include all modules.
    foreach ($this->getModules() as $module => $module_info) {
      $filename = $module_info->getPathname();

      // Match for custom modules.
      if ($this->includeCustomModules() && preg_match($this->getCustomRegex(), $filename)) {
        $result['custom'] = array_merge($result['custom'], $this->getModuleDetails($module, $filename));
      }

      // Match for contrib modules.
      if ($this->includeContribModules() && preg_match($this->getContribRegex(), $filename)) {
        $result['contrib'] = array_merge($result['contrib'], $this->getModuleDetails($module, $filename));
      }
    }

    return $result;
  }

  /**
   * Update Warden with latest site data.
   *
   * @throws \Exception
   *   If any problems occur.
   */
  public function updateWarden() {
    $data = $this->generateSiteData();
    $this->postSiteData($data);
  }

  /**
   * Get the local token and generate it if it is not set.
   *
   * @return string
   *   The local token
   */
  public function getLocalToken() {
    if (!isset($this->localToken)) {
      $this->localToken = $this->generateNewLocalToken();
    }

    return $this->localToken;
  }

  /**
   * Generate and save and new token.
   *
   * @return string
   */
  protected function generateNewLocalToken() {
    $local_token = hash('sha256', mt_rand());
    \Drupal::configFactory()
      ->getEditable('warden.settings')
      ->set('warden_token', $local_token)
      ->save();
    return $local_token;
  }

  /**
   * @param string $module
   *   The name of the module
   * @param string $filename
   *   A full path to an info yml file.
   *
   * @return array
   *   key is a project name or module name
   *   value is an array of its details
   */
  protected function getModuleDetails($module, $filename) {
    $details = $this->getInfoParser()->parse($filename);
    if (!empty($details['project'])) {
      return [
        $details['project'] => [
          'version' => $details['version'],
        ]
      ];
    }
    else {
      return [
        $module => [
          'version' => $details['version'],
        ]
      ];
    }
  }

  /**
   * @return string
   */
  protected function getSiteName() {
    return $this->siteName;
  }

  /**
   * @return string
   */
  protected function getCoreVersion() {
    return $this->coreVersion;
  }

  /**
   * @return string
   *   The regex telling us where a contrib module is located.
   */
  protected function getContribRegex() {
    return $this->contribModuleRegex;
  }

  /**
   * @return string
   *   The regex telling us where a custom module is located.
   */
  protected function getCustomRegex() {
    return $this->customModuleRegex;
  }

  /**
   * @return bool
   */
  protected function includeCustomModules() {
    return $this->includeCustom;
  }

  /**
   * @return bool
   */
  protected function includeContribModules() {
    return $this->includeContrib;
  }

  /**
   * @param Extension[] $themes
   * @return $this
   */
  public function setThemes(array $themes) {
    $this->themes = $themes;
    return $this;
  }

  /**
   * @return \Drupal\Core\Extension\Extension[]
   */
  protected function getThemes() {
    if (!isset($this->themes)) {
      $this->setThemes(system_list('theme'));
    }

    return $this->themes;
  }

  /**
   * @param Extension[] $modules
   * @return $this
   */
  public function setModules(array $modules) {
    $this->modules = $modules;
    return $this;
  }

  /**
   * @return Extension[]
   */
  protected function getModules() {
    if (!isset($this->modules)) {
      $listing = new ExtensionDiscovery(\Drupal::root());
      $this->setModules($listing->scan('module'));
    }

    return $this->modules;
  }

  /**
   * @return \Drupal\Core\Extension\InfoParser
   */
  protected function getInfoParser() {
    if (empty($this->infoParser)) {
      $this->setInfoParser(new InfoParser());
    }

    return $this->infoParser;
  }

  /**
   * @param \Drupal\Core\Extension\InfoParser $infoParser
   * @return $this
   */
  public function setInfoParser(InfoParser $infoParser) {
    $this->infoParser = $infoParser;
    return $this;
  }

  /**
   * @return string
   */
  protected function getBaseUrl() {
    if (!isset($this->baseUrl)) {
      global $base_url;
      $this->setBaseUrl($base_url);
    }

    return $this->baseUrl;
  }

  /**
   * @param string $baseUrl
   * @return $this
   */
  public function setBaseUrl($baseUrl) {
    $this->baseUrl = $baseUrl;
    return $this;
  }

  /**
   * @return int
   */
  protected function getTime() {
    if (!isset($this->time)) {
      $this->setTime(time());
    }

    return $this->time;
  }

  /**
   * @param int $time
   * @return $this
   */
  public function setTime($time) {
    $this->time = $time;
    return $this;
  }

}
